#include <stdio.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <wchar.h>
#include <ctype.h>
#include <malloc.h>
#include <memory.h>

#include <windows.h>
#include <crtdbg.h>

#include <QubFile.h>

#include <Model.h>
#include <Bone.h>
#include <MotionPoint.h>
#include <quaternion.h>

#define DBGH
#include <dbg.h>

#include <string>

CQubFile::CQubFile()
{
	InitParams();
}

CQubFile::~CQubFile()
{
	DestroyObjs();
}

int CQubFile::InitParams()
{
	CXMLIO::InitParams();

	m_topbone = 0;
	m_bonenum = 0;
	m_motid = -1;
	m_motinfo = 0;

	return 0;
}

int CQubFile::DestroyObjs()
{
	CXMLIO::DestroyObjs();

	InitParams();

	return 0;
}


int CQubFile::WriteQubFile( WCHAR* strpath, MOTINFO* srcmotinfo, CBone* srctopbone, int srcbonenum )
{
	m_topbone = srctopbone;
	m_motinfo = srcmotinfo;
	if( srcmotinfo ){
		m_motid = srcmotinfo->motid;
	}else{
		m_motid = -1;
		_ASSERT( 0 );
		return 1;
	}
	m_bonenum = srcbonenum;
	m_mode = XMLIO_WRITE;

	m_hfile = CreateFile( strpath, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_ALWAYS,
		FILE_FLAG_SEQUENTIAL_SCAN, NULL );
	if( m_hfile == INVALID_HANDLE_VALUE ){
		DbgOut( L"QubFile : WriteQubFile : file open error !!!\n" );
		_ASSERT( 0 );
		return 1;
	}


	CallF( Write2File( "<?xml version=\"1.0\" encoding=\"Shift_JIS\"?>\r\n<QUB>\r\n" ), return 1 );  


	CallF( WriteFileInfo(), return 1 );

	WriteBoneReq( m_topbone, 0 );


	CallF( Write2File( "</QUB>\r\n" ), return 1 );

	return 0;
}

int CQubFile::WriteFileInfo()
{

	CallF( Write2File( "  <FileInfo>\r\n    <kind>OpenRDBMotionFile</kind>\r\n    <version>1001</version>\r\n    <type>0</type>\r\n  </FileInfo>\r\n" ), return 1 );

	CallF( Write2File( "  <MOTIONINFO>\r\n" ), return 1 );
	CallF( Write2File( "    <Name>%s</Name>\r\n", m_motinfo->motname ), return 1 );
	CallF( Write2File( "    <FrameLeng>%f</FrameLeng>\r\n", m_motinfo->frameleng ), return 1 );
	CallF( Write2File( "    <Loop>%d</Loop>\r\n", m_motinfo->loopflag ), return 1 );
	CallF( Write2File( "    <BoneNum>%d</BoneNum>\r\n", m_bonenum ), return 1 );

	CallF( Write2File( "  </MOTIONINFO>\r\n" ), return 1 );

	return 0;
}

void CQubFile::WriteBoneReq( CBone* srcbone, int broflag )
{
	CallF( Write2File( "  <Bone>\r\n" ), return );

	int mpnum = 0;
	int mpcnt = 0;
	CMotionPoint* curmp;	
	curmp = srcbone->m_motionkey[ m_motid ];
	while( curmp ){
		mpcnt++;
		curmp = curmp->m_next;
	}
	mpnum = mpcnt;

	CallF( Write2File( "    <Name>%s</Name>\r\n", srcbone->m_bonename ), return );
	CallF( Write2File( "    <MPNum>%d</MPNum>\r\n", mpnum ), return );


	mpcnt = 0;
	curmp;	
	curmp = srcbone->m_motionkey[ m_motid ];
	while( curmp ){
		mpcnt++;

		CallF( Write2File( "      <MotionPoint>\r\n" ), return );
		CallF( Write2File( "        <Frame>%.7f</Frame>\r\n", curmp->m_frame ), return );
		CallF( Write2File( "        <Q>%f, %f, %f, %f</Q>\r\n", curmp->m_q.x, curmp->m_q.y, curmp->m_q.z, curmp->m_q.w ), return );
		CallF( Write2File( "        <Eul>%f, %f, %f</Eul>\r\n", curmp->m_eul.x, curmp->m_eul.y, curmp->m_eul.z ), return );
		CallF( Write2File( "        <Tra>%f, %f, %f</Tra>\r\n", curmp->m_tra.x, curmp->m_tra.y, curmp->m_tra.z ), return );
		CallF( Write2File( "      </MotionPoint>\r\n" ), return );

		curmp = curmp->m_next;
	}

	if( mpcnt != mpnum ){
		_ASSERT( 0 );
		return;
	}

	CallF( Write2File( "  </Bone>\r\n" ), return );

	if( srcbone->m_child ){
		WriteBoneReq( srcbone->m_child, 1 );
	}

	if( broflag && srcbone->m_brother ){
		WriteBoneReq( srcbone->m_brother, 1 );
	}
}

int CQubFile::LoadQubFile( WCHAR* strpath, CModel* srcmodel, int* newid )
{
	m_model = srcmodel;
	m_motid = -1;
	m_bonenum = 0;
	m_mode = XMLIO_LOAD;

	WCHAR wfilename[MAX_PATH] = {0L};
	WCHAR* lasten;
	lasten = wcsrchr( strpath, TEXT('\\') );
	if( !lasten ){
		_ASSERT( 0 );
		return 1;
	}
	wcscpy_s( wfilename, MAX_PATH, lasten + 1 );

	m_hfile = CreateFile( strpath, GENERIC_READ, 0, NULL, OPEN_EXISTING,
		FILE_FLAG_SEQUENTIAL_SCAN, NULL );
	if( m_hfile == INVALID_HANDLE_VALUE ){
		_ASSERT( 0 );
		return 1;
	}	

	CallF( SetBuffer(), return 1 );

	int posstep = 0;
	XMLIOBUF fileinfobuf;
	ZeroMemory( &fileinfobuf, sizeof( XMLIOBUF ) );
	CallF( SetXmlIOBuf( &m_xmliobuf, "<FileInfo>", "</FileInfo>", &fileinfobuf ), return 1 );
	CallF( CheckFileVersion( &fileinfobuf ), return 1 );

	XMLIOBUF motinfobuf;
	ZeroMemory( &motinfobuf, sizeof( XMLIOBUF ) );
	CallF( SetXmlIOBuf( &m_xmliobuf, "<MOTIONINFO>", "</MOTIONINFO>", &motinfobuf ), return 1 );
	CallF( ReadMotionInfo( wfilename, &motinfobuf ), return 1 );

	int bonecnt;
	for( bonecnt = 0; bonecnt < m_bonenum; bonecnt++ ){
		XMLIOBUF bonebuf;
		ZeroMemory( &bonebuf, sizeof( XMLIOBUF ) );
		CallF( SetXmlIOBuf( &m_xmliobuf, "<Bone>", "</Bone>", &bonebuf ), return 1 );
		CallF( ReadBone( &bonebuf ), return 1 );
	}

	*newid = m_motid;

	return 0;
}


int CQubFile::CheckFileVersion( XMLIOBUF* xmlbuf )
{
	char kind[256];
	char version[256];
	char type[256];
	ZeroMemory( kind, sizeof( char ) * 256 );
	ZeroMemory( version, sizeof( char ) * 256 );
	ZeroMemory( type, sizeof( char ) * 256 );

	CallF( Read_Str( xmlbuf, "<kind>", "</kind>", kind, 256 ), return 1 );
	CallF( Read_Str( xmlbuf, "<version>", "</version>", version, 256 ), return 1 );
	CallF( Read_Str( xmlbuf, "<type>", "</type>", type, 256 ), return 1 );

	int cmpkind, cmpversion, cmptype;
	cmpkind = strcmp( kind, "OpenRDBMotionFile" );
	cmpversion = strcmp( version, "1001" );
	cmptype = strcmp( type, "0" );

	if( (cmpkind == 0) && (cmpversion == 0) && (cmptype == 0) ){
		return 0;
	}else{
		_ASSERT( 0 );
		return 1;
	}

	return 0;
}
int CQubFile::ReadMotionInfo( WCHAR* wfilename, XMLIOBUF* xmlbuf )
{
	char motionname[256] = {0};
	float fmotionleng = 0.0f;
	int loopflag = 0;

	CallF( Read_Str( xmlbuf, "<Name>", "</Name>", motionname, 256 ), return 1 );
	CallF( Read_Float( xmlbuf, "<FrameLeng>", "</FrameLeng>", &fmotionleng ), return 1 );
	CallF( Read_Int( xmlbuf, "<Loop>", "</Loop>", &loopflag ), return 1 );
	CallF( Read_Int( xmlbuf, "<BoneNum>", "</BoneNum>", &m_bonenum ), return 1 );

	CallF( m_model->AddMotion( motionname, wfilename, (double)fmotionleng, &m_motid ), return 1 );
	m_motinfo = m_model->m_motinfo[ m_motid ];
	if( !m_motinfo ){
		_ASSERT( 0 );
		return 1;
	}
	m_motinfo->loopflag = loopflag;

	return 0;
}
int CQubFile::ReadBone( XMLIOBUF* xmlbuf )
{
//    <Name>bonename1</Name>
//    <MPNum>2</MPNum>

	char bonename[256] = {0};
	int mpnum = 0;

	CallF( Read_Str( xmlbuf, "<Name>", "</Name>", bonename, 256 ), return 1 );
	CallF( Read_Int( xmlbuf, "<MPNum>", "</MPNum>", &mpnum ), return 1 );

	CBone* curbone = m_model->m_bonename[ bonename ];

	if( mpnum > 0 ){
		int mpcnt;

		for( mpcnt = 0; mpcnt < mpnum; mpcnt++ ){
			XMLIOBUF mpbuf;
			ZeroMemory( &mpbuf, sizeof( XMLIOBUF ) );
			CallF( SetXmlIOBuf( xmlbuf, "<MotionPoint>", "</MotionPoint>", &mpbuf ), return 1 );
			if( curbone ){
				CallF( ReadMotionPoint( &mpbuf, curbone ), return 1 );
			}
		}
	}

	return 0;
}
int CQubFile::ReadMotionPoint( XMLIOBUF* xmlbuf, CBone* srcbone )
{
	float fframe = 0.0f;
	CQuaternion q;
	D3DXVECTOR3 eul, tra;

	CallF( Read_Float( xmlbuf, "<Frame>", "</Frame>", &fframe ), return 1 );
	CallF( Read_Q( xmlbuf, "<Q>", "</Q>", &q ), return 1 );
	CallF( Read_Vec3( xmlbuf, "<Eul>", "</Eul>", &eul ), return 1 );
	CallF( Read_Vec3( xmlbuf, "<Tra>", "</Tra>", &tra ), return 1 );

	CMotionPoint* newmp = 0;
	int existflag = 0;
	newmp = srcbone->AddMotionPoint( m_motid, (double)fframe, 0, &existflag );
	if( !newmp ){
		_ASSERT( 0 );
		return 1;
	}

//DbgOut( L"readmp : bone %s, frame %f, q(%f, %f, %f, %f)\r\n",
//	srcbone->m_wbonename, fframe, q.x, q.y, q.z, q.w );

	newmp->m_q = q;
	newmp->m_eul = eul;
	newmp->m_tra = tra;

/***
    <MotionPoint>
      <Frame>0.0</Frame>
      <Q>0.0, 0.0, 0.0, 1.0</Q>
      <Eul>0.0, 0.0, 0.0</Eul>
      <Tra>0.0, 0.0, 0.0</Tra>
    </MotionPoint>
***/
	return 0;
}

